;---------------------------------------
;Fun With CBMPRGStudio - Part 4
;Scene World issue #35
;
;Enhanced version of the Scene World
;intro.
;
;Code, graphics and music by RICHARD
;---------------------------------------

;Variables

;Declare screen RAM as a variable
;to point at default screen RAM
;$0400

screen = $0400
colour = $d800
char1x1mem = $3000

;Declare matrix as a variable 
;to point at $2800. Where the 
;matrix file is imported to. 

matrix = $2800 

;Declare raster split position as
;five variables

split1 = $2a
split2 = $82
split3 = $aa
split4 = $c8
split5 = $fa


;Declare music init and play addresses

musicinit = $1000
musicplay = $1003


;Declare start address of linked prg

startaddress = $fce2 ;soft reset

;Set target in CBMPRGSTUDIO 
;to Commodore 64

TGT_C64

;Insert BASIC SYS run address for 
;program at 2064 ($0810)

; 10 SYS2064

        *=$0801

        BYTE $0B,$08,$0A,$00,$9E
        BYTE $32,$30,$36,$34,$00
        BYTE $00,$00

;Code starts here

        *=$0810

        sei

;Detect PAL/NTSC
        
        jsr systemdetect

;One-time invert charset (since the
;intro is being run once after 
;compiling)

invloop ldx #$00
invert1 lda char1x1mem,x
        eor #$ff
invert2 sta char1x1mem,x 
        inx
        bne invert1
        inc invert1+2
        inc invert2+2
        lda invert1+2
        cmp #$38
        bne invloop
        
;Set border and background colour to
;black.

        lda #$00
        sta $d020
        sta $d021

        sta case 
        
        ;Char multicolour 1 = dark blue
        ldx #$06
        ;Char multicolour 2 = light blue
        ldy #$0e
        stx $d022 
        sty $d023
        
;Fill screen RAM with the space char
;inside the screen RAM.

        ldx #$00
clear   lda #$20
        sta screen,x
        sta screen+$100,x
        sta screen+$200,x
        sta screen+$2e8,x

        ;Multicolour cyan
        lda #$00
        sta colour,x
        sta colour+$100,x
        sta colour+$200,x
        sta colour+$2e8,x
        inx
        bne clear

;Draw the logo on to the screen 
;two rows after the starting row and
;40 columns across, 8 rows down.

        ldx #$00
drawlogo
        lda matrix,x
        sta screen,x
        lda matrix+40,x
        sta screen+40,x
        lda matrix+80,x
        sta screen+80,x
        lda matrix+120,x
        sta screen+120,x
        lda matrix+160,x
        sta screen+160,x
        lda matrix+200,x
        sta screen+200,x
        lda matrix+240,x
        sta screen+240,x
        lda matrix+280,x
        sta screen+280,x
        lda matrix+320,x
        sta screen+320,x
        lda matrix+360,x
        sta screen+360,x
        
        inx
        cpx #40
        bne drawlogo

;Paint the logo multicolour chars cyan
        
       ldx #$00
filllogo
       lda #$0b
       sta colour,x
       sta colour+40,x
       sta colour+80,x
       sta colour+120,x 
       sta colour+160,x
       sta colour+200,x
       sta colour+240,x
       sta colour+280,x
       sta colour+320,x
       lda #0
       sta colour+360,x
       sta colour+400,x
       inx
       cpx #40
       bne filllogo
; 
;Place presentation lines on to the
;screen RAM, two rows after the logo
;matrix.

        ldx #$00
drawlines
        lda line1,x
        sta screen+440,x
        lda line2,x
        sta screen+520,x

        ;Black out lines and scroll
        ;line. 

        lda #0
        sta colour+440,x
        sta colour+520,x
        sta colour+640,x

        inx
        cpx #40
        bne drawlines

        ;Initialize the flash pointers

        lda #0
        sta fadedelay
        sta fadepointer
        sta fadestore

;Initialize the scrolling message

        lda #<scrolltext
        sta messread+1
        lda #>scrolltext
        sta messread+2
        
;Setup the sprite settings


; - Enable, all sprites on,
;   sprite priority, multicolour mode

        lda #$ff
        sta $d015
        sta $d01b
        sta $d01c

;Disable sprite X/Y expansion size

        lda #$00
        sta $d017
        sta $d01d
        
;Set green scheme for intro sprites 
;also set starting frame for intro
;sprites.

        lda #$0d
        ldy #$01
        sta $d025
        sty $d026
        ldx #$00
greensp lda #$05
        sta $d027,x
        lda #$b0
        sta $07f8,x
        lda #$94 ;First frame at $2500
        inx
        cpx #$08
        bne greensp


;Initialize and setup IRQ raster 
;interrupt.

        ldx #<irq1
        ldy #>irq1
        lda #$7f
        stx $0314
        sty $0315
        sta $dc0d
        sta $dd0d
        lda #$2a
        sta $d012
        lda #$1b
        sta $d011
        lda #$01
        sta $d01a
        
        ;Initialize music
        lda #$00
        jsr musicinit

        cli

;Main intro loop 
        
introloop
        ;Subr. synchronize timer
        jsr synctimer
        ;Expand sprite MSB 
        jsr spritemsb
        ;Subr. scroller 
        jsr scroller
        ;Subr. colourrtn (colour flash)
        jsr colourrtn
        ;Wave routine 
        jsr waves
        ;Scroll raster colours
        jsr barscroll
        ;Scroll text bounce routine
        jsr bouncer
        ;Animate sprites
        jsr animation
        ;Wait for spacebar
        lda $dc01
        cmp #$ef
        beq exitintro
        jmp introloop

;Intro has finished. Exit the intro,
;switch off all IRQs and call relocator

exitintro
        sei

        ;Kill IRQ interrupts

        lda #$00
        sta $d019
        sta $d01a
        lda #$81
        sta $dc0d
        sta $dd0d
        lda #$1b
        sta $d011
        ldx #$31
        ldy #$ea
        stx $0314
        sty $0315
        cli

        ;Init with C64 blue screen
        jsr $ff81
        lda #0
        sta $d020
        sta $d021

        ;Clear SID chip

        ldx #$00
silence
        lda #$00
        sta $d400,x
        inx
        cpx #$18
        bne silence

        ;Copy transfer data to low mem

        ldx #$00
copytrs
        lda relocator,x
        sta $0400,x
        lda #0
        sta $d800,x
        inx
        bne copytrs

        lda #$00
        sta $0800
        cli
        jmp $0400

;Main relocate subroutine

relocator
        sei
        lda #$34
        sta $01
        
reloc1  ldx #$00 
reloc2  lda $4800,x ;linked prg pos
        sta $0801,x ;basic ram pos
        inx
        bne reloc2
        inc $0409
        inc $040c
        lda $0409
        bne reloc1
        lda #$37
        sta $01
        cli
        jsr $a659
        jmp $a7ae

;Synchronize raster timer (rt) with
;main intro loop and IRQ raster 
;interrupt

synctimer
        lda #0
        sta rt
        cmp rt
        beq *-3
        rts

;Expand the screen area for positioning
;sprites

spritemsb
        ldx #$00
msbloop lda starpos+1,x ;Stars pos Y
        sta $d001,x ;HW Sprite pos Y
        lda starpos,x 
        asl         ;Double the size
        ror $d010   ;of the X position
        sta $d000,x
        inx         ;Increment pos Y
        inx         ;the increment pos X
        cpx #16     ;Total sprites 
        bne msbloop ;position altogether
        rts

;2x2 Scroller routine


scroller
        lda xpos
        sec
        sbc #2 ;Scroll speed
        and #7 ;BITS 0-8
        sta xpos
        bcs exitscroll

        ;Shift screen mem for text

        ldx #$00
shift   lda screen+641,x
        sta screen+640,x
        inx
        cpx #$50 ;79 chars
        bne shift

        ;Record scroll read position

messread
        lda scrolltext
        cmp #$00
        bne store
        
        ;@ detected, reset scroll text

        lda #<scrolltext
        sta messread+1
        lda #>scrolltext
        sta messread+2
        jmp messread

store   sta char ;Store mess.char
        lda case  ;Check case mode
        cmp #1
        beq shiftedchar
        ;Normal char mode
        lda char
        sta screen+679
        ;Reverse char mode
        eor #$80
        sta screen+719
        lda #1
        sta case
exitscroll
        rts
shiftedchar
        ;Shifted char mode
        lda char 
        eor #$40
        sta screen+679
        ;Reverse char mode
        eor #$80
        sta screen+719
        ;Reset char case type
        lda #0
        sta case
        ;Update to next character
        inc messread+1
        bne exitscroll
        inc messread+2
        jmp exitscroll



;Colour flash and washing routine

colourrtn
        lda fadedelay
        cmp #1 ;Duration of fade
        beq fademain
        inc fadedelay
        rts

        ;Main colour fade routine

fademain
        lda #0 ;reset fade delay 
        sta fadedelay ;pointer
        
        ldx fadepointer
        lda colourtable,x
        sta fadestore
        inx
        cpx #$27 ;40 bytes read?
        beq looptable
        inc fadepointer ;next byte
        jmp storecols

        ;Reset table read 
looptable
        ldx #0
        stx fadepointer

        ;Main colour washing/flashing
        ;routines.

storecols

    ;1. wash colour from right of
    ;screen to left for top text line

        lda fadestore
        sta colour+679
        sta colour+719
        ;Move colour backwards

        ldx #$00
shiftback
        lda colour+641,x
        sta colour+640,x
        lda colour+681,x
        sta colour+680,x
        inx
        cpx #$28 ;40 bytes read
        bne shiftback
        rts

        
;Wave routine
waves 
        lda wavetable+79;Read last byte
        sta wavstore ;Store to a zeropage
                ;as a temp byte 

        ldx #78
shiftwave
        lda wavetable,x
        sta wavetable+1,x
        dex
        bpl shiftwave
        lda wavstore
        sta wavetable
        rts

 ; Raster bar scrolling routine
barscroll
        lda coldelay ;Read delay 
        cmp #3       ;Speed of scroll
        beq dobarscroll ; == 3
        inc coldelay ;Else read next
        rts

        ;Main bar scrolling
dobarscroll
        lda #0
        sta coldelay ;Reset scrolldelay
        lda rastercol ;Get first byte 
        sta colstore1 ;of raster scroll
                      ;table.
        lda rastercol+31 ;Get last byte 
        sta colstore2 ;of raster scroll
                      ;table

        ;Scroll top half of raster 
        ;colours

        ldx #$00
scrolltop
        lda rastercol+1,x ;Read +1 byte 
        sta rastercol,x ;Pull it back 
        inx
        cpx #15
        bne scrolltop 
        lda colstore1
        sta rastercol+14
        
        ;Scroll bottom half of raster
        ;colours 

        ldx #15
scrollbtm
        lda rastercol+16,x ;Read byte
        sta rastercol+17,x;Push forward
        dex
        bpl scrollbtm 
        lda colstore2 
        sta rastercol+16
        rts

;FLD Bounce scroll text subroutine 
bouncer 
        ldx bouncepointer
        lda SINUS,x
        sta fldsm+1
        inx
        cpx #127
        beq resetbounce
        inc bouncepointer
        rts
resetbounce
        ldx #0
        stx bouncepointer
        rts

;Animate the star sprites 

        
animation
        ;Process the sprite animation
        ;speed.
        lda animspeed
        cmp #4 ;<-- Delay rate set
        beq dospranim
        inc animspeed
        rts
dospranim
        ;Reset animspeed 
        lda #0
        sta animspeed

        
        ldx #$00
animloop
        ;Read all 8 bytes of the 
        ;frame table and place in 
        ;all the default sprite type
        ;$07f8-$07ff

        lda frametable,x
        sta $07f8,x
        inx
        cpx #$08 ;8 sprites done?
        bne animloop 

        ;Modify the animation table 
        ;by moving to the next value
        ;in each frame.

        lda frametable
        sta frametemp
        ldx #$00
shifttable
        lda frametable+1,x
        sta frametable,x
        inx
        cpx #9
        bne shifttable
        lda frametemp
        sta frametable+9
        rts
        
        

;Setup IRQ raster interrupt - Still
;screen, and synchronize raster timer
;(rt). 

irq1    inc $d019
        lda $dc0d
        sta $dd0d
        lda #split1
        sta $d012

      

        lda #$1c
        sta $d011
        lda #1
        sta rt 
 
        ;Test colour BLACK

      ; lda #0
      ; sta $d020


        ;Play music
        jsr musicplayer

        ;Point to next IRQ raster 
        ;interrupt

        
        ldx #<irq2
        ldy #>irq2
        stx $0314
        sty $0315
        jmp $ea7e

;Setup IRQ raster interrupt 2 - 
;Logo matrix position

irq2    inc $d019
        lda #split2
        sta $d012
        ldy $d012
     
        ;Setup screen multicolour mode
        ;for logo.

        lda #$18
        sta $d016

        ;Set charset mode at $2000 
        ;(where logo charset is placed)

        lda #$18
        sta $d018
        
        ;Test colour WHITE

      ;  lda #1
      ;  sta $d020

        ; Move raster down with wobble
        ; table being read. 

        ldy $d012 
        ldx #79   
wavloop lda wavetable,x
        cpy $d012 
        beq *-3   
        sta $d016
        iny
        dex
        bpl wavloop 

        ;Stop wobbling here
        lda #$18
        sta $d016

        ;Time out rasters slightly
        ldx #$08
        dex
        bne *-1
        
       

        ;Point to next IRQ raster 
        ;interrupt 

        ldx #<irq3
        ldy #>irq3
        stx $0314
        sty $0315
        jmp $ea7e

;Setup IRQ raster interrupt 3 -
;Still text 

irq3    inc $d019
        lda #split3
        sta $d012

        ;Set char multicolour mode 
        ;to off

        lda #$08
        sta $d016

        ;Set charset mode for 1x1
        ;character set which is at
        ;$3000

        lda #$1c
        sta $d018

        ;Create raster bar behind the
        ;presentation text 

        ldy $d012
        ldx #31 ;Start from last byte 
rcloop  lda rastercol,x
        cpy $d012
        beq *-3
        sta $d021
        iny
        dex
        bpl rcloop
        lda #0
        sta $d021
        ldx #$08
        dex
        bne *-1
        
        

      ; lda #2
      ; sta $d020

        ldx #<irq4
        ldy #>irq4
        stx $0314
        sty $0315
        jmp $ea7e

;Setup IRQ raster interrupt 4 -
;Smooth scroll text 

irq4    inc $d019
        lda #split4
        sta $d012

        ;Set char scroll byte to 
        ;screen horizontal position 
        ;for smooth scroll hires mode

        lda xpos
        sta $d016

        ;Set 2x2 charset based at 
        ;$3800

        lda #$1e
        sta $d018
       

fldsm   ldx #$88
bounce  ldy $d012
        dey
        tya
        and #$07
        ora #$10
        sta $d011
        dex
        bne bounce

      ;  lda #3
      ;  sta $d020

        ldx #<irq5
        ldy #>irq5
        stx $0314
        sty $0315
        jmp $ea7e

;Setup IRQ raster interrupt 5 -
;Still screen again.

irq5    inc $d019
        lda #split5
        sta $d012

       
        ;Set char multicolour off

;        lda #$08
;        sta $d016

        ;Test border
        ;lda #$04
        ;sta $d020
        
        ;Jump back to IRQ 1

        ldx #<irq1
        ldy #>irq1
        stx $0314
        sty $0315
        jmp $ea7e 

;One time system detection PAL/NTSC 
systemdetect
        lda $d012 
        cmp $d012
        beq *-3
        bmi systemdetect
        cmp #$22
        bcc ntsc
        lda #1
        sta system
        rts 
ntsc    lda #0
        sta system
        rts 
        ;NTSC system found
ntscfound
        lda #$00
        sta system
        rts

;PAL/NTSC based music player
musicplayer
        lda system 
        cmp #1
        beq pal
        inc ntsctimer
        lda ntsctimer
        cmp #$06 
        beq resetntsc
pal     jsr musicplay
        rts 
resetntsc 
          
        lda #0
        sta ntsctimer
        rts


;System PAL/NTSC
system  byte 0

;NTSC music timer
ntsctimer byte 0

xpos    byte 0

;Sync raster time control byte 

rt      byte 0

;Colour fade pointers

fadedelay 
        byte 0
fadepointer
        byte 0
fadestore
        byte 0
wavstore 
        byte 0

;Raster colour pointers

coldelay byte 0 ;Raster roll delay
colstore1 byte 0 ;Raster tempbyte 1
colstore2 byte 0 ;Raster tempbyte 2

;2x2 scroll text pointers
case byte 0 ;Char type normal/shifted
char byte 0 ;Message char stored

;FLD bounce pointer
bouncepointer byte 0

;Sprite animation speed pointer
animspeed byte 0
;Sprite frame temporary store pointer
frametemp byte $b0

;Sprite starting frame for animation 
frametable byte $b0,$b1,$b2,$b3,$b4,$b5
           byte $b6,$b7,$b8,$b9

;Star sprites X/Y positioning table

starpos byte $10,$b0 ;Sprite 0 X/Y
        byte $40,$c0 ;Sprite 1 X/Y
        byte $70,$d0 ;Sprite 2 X/Y
        byte $9c,$e0 ;Sprite 3 X/Y
        byte $88,$b8 ;Sprite 4 X/Y
        byte $58,$e8 ;Sprite 5 X/Y
        byte $28,$d8 ;Sprite 6 X/Y
        byte $9c,$c8 ;Sprite 7 X/Y



;Wave table
wavetable

 byte $11,$12,$12,$13,$13,$13,$14,$14
 byte $14,$14,$15,$15,$15,$15,$16,$16
 byte $16,$16,$16,$16,$16,$17,$17,$17
 byte $17,$17,$17,$17,$17,$17,$17,$17
 byte $17,$17,$17,$17,$16,$16,$16,$16
 byte $16,$16,$15,$15,$15,$15,$15,$15
 byte $14,$14,$14,$14,$13,$13,$13,$12
 byte $12,$11,$11,$11,$11,$11,$11,$11
 byte $11,$11,$11,$11,$11,$11,$11,$11
 byte $11,$11,$11,$11,$11,$11,$11,$11

;Raster colour table (32 bytes)

rastercol

 byte $0e,$03,$0d,$01,$0d,$03,$0e,$06
 byte $08,$0a,$07,$01,$07,$0a,$08,$02
 byte $08,$0a,$07,$01,$07,$0a,$08,$02
 byte $0e,$03,$0d,$01,$0d,$03,$0e,$06
;Colour table
 align $100
colourtable
 byte $0e,$03,$0d,$01,$0d,$03,$0e,$06
 byte $08,$0a,$07,$01,$07,$0a,$08,$02
 byte $0c,$0f,$03,$01,$03,$0f,$0c,$0b
 byte $02,$0a,$07,$01,$07,$0a,$08,$02
 byte $0e,$03,$0d,$01,$0d,$03,$0e,$06

 align $100

SINUS byte 3,2,2,1,1,1
      byte 2,2,4,5,7,9
      byte 11,13,16,19,23,26
      byte 30,34,39,43,48,53
      byte 58,63,69,75,80,86
      byte 92,98,104,110,117,123
      byte 128,122,115,109,103,97
      byte 91,85,79,74,68,63
      byte 57,52,47,42,38,34
      byte 30,26,22,19,16,13
      byte 10,8,6,5,3,2
      byte 2,1,1,1,2,2
      byte 4,5,7,9,11,13
      byte 16,19,23,26,30,34
      byte 39,43,48,53,58,63
      byte 69,75,80,86,92,98
      byte 104,110,117,123,128,122
      byte 115,109,103,97,91,85
      byte 79,74,68,63,57,52
      byte 47,42,38,34,30,26
      byte 22,19,16,13,10,8
      byte 6,5

;2 line presentation text

line1 text '            proudly '
      text 'presents            '
line2 text '         scene world'
      text ' - issue 35         '

;---------------------------------------

;Include music data
        *=$1000
        incbin "c64/music.prg",2,0

;---------------------------------------

;Include logo charset data
        *=$2000
        incbin "c64/logo_charsetv2.bin"

;---------------------------------------

;Include logo matrix data
        *=$2800
        incbin "c64/logo_matrixv2.bin"

;---------------------------------------
;Include animated sprite data
        *=$2c00
        incbin "c64/sprites.bin"

;---------------------------------------

;Include 1x1 charset data
        *=$3000
        incbin "c64\charset.bin"

;---------------------------------------
;Include 2x2 charset data 
        *=$3800
        incbin "c64\2x2charset.prg",2,0

;---------------------------------------

;Scroll text message
        *=$4000
scrolltext
 text ' ... welcome to issue 35 of sce'
 text 'ne world ...   the most ecelecti'
 text 'c and the only commodore 64 pal '
 text 'and ntsc disk mag in the world '
 text '...   this issue is the winter '
 text 'edition of the diskmag, and we '
 text 'surely have some great stuff in '
 text 'store for you once more ...   '
 text 'there is the usual games, demo '
 text 'scene news, views, interviews '
 text 'and reports ...   not only that '
 text '...  scene world continues its '
 text 'programming venture and shows ' 
 text 'you how this intro is made ...  '
 text 'it was made as part of my "fun '
 text 'with cbmprgstudio" feature' ... '
 text '  this intro was programmed by '
 text 'richard, who also did the charse'
 text 't and the music ...   the cool '
 text 'logo was drawn by shine ...   '
 text 'i was originally thinking about making '
 text 'this intro christmas themed, but with '
 text 'the uncertainty of getting this issue '
 text 'done in time for christmas, i decided not to make '
 text 'a christmas themed intro at the end ...    '
 text 'i would like to say a huge thank you to the scene '
 text 'world team, and its contributors for their kind '
 text 'submissions ...   also a huge thank you goes '
 text 'to shine, for kindly contributing his logo '
 text 'specially for this intro ...   also to arthur jordison for the amazing cbmprgstudio '
 text 'c64 development suite ...   '
 text 'scene world sends its greetings '
 text 'to the following c64 indie gaming '
 text 'and scene mags today '
 text ' ...   they are as follows: '
 text 'attitude, commodore format archive, '
 text 'propaganda, vandalism news, rapid news, '
 text 'remeta, digital talk, reset 64, k and a plus, '
 text 'zzap! 64 micro action, indieretronews, '
 text 'and vintage is the new old ...   also greetings '
 text 'fly out to all the active c64 games developers, '
 text 'demo sceners and c64 fans still keeping the '
 text 'c64 spirit active today ...   okay, '
 text 'that is enough from me, until next issue it is '
 text 'goodbye for now ...   
 text '                                              '
  byte 0

;--------------------------------------

       ; *=$4800 ;Linked program here
       ; incbin "c64\testprg.prg",2,0

;--------------------------------------